@@ -1,5 +1,5 @@
 /* SPDX-License-Identifier: GPL-2.0 */
-/* Copyright (C) 2009-2020  B.A.T.M.A.N. contributors:
+/* Copyright (C) B.A.T.M.A.N. contributors:
  *
  * Marek Lindner <mareklindner@neomailbox.ch>
  *
@@ -18,18 +18,9 @@
 #include "batman_adv.h"
 #include "netlink.h"
 
-#define SYS_BATIF_PATH_FMT	"/sys/class/net/%s/mesh/"
-#define SYS_IFACE_PATH		"/sys/class/net"
-#define SYS_IFACE_DIR		SYS_IFACE_PATH"/%s/"
-#define SYS_HARDIF_PATH		SYS_IFACE_DIR "batman_adv/"
-#define SYS_MESH_IFACE_FMT	SYS_HARDIF_PATH "mesh_iface"
-#define SYS_IFACE_STATUS_FMT	SYS_HARDIF_PATH "iface_status"
-#define SYS_VLAN_PATH		SYS_IFACE_PATH"/%s/mesh/vlan%d/"
-#define SYS_ROUTING_ALGO_FMT	SYS_IFACE_PATH"/%s/mesh/routing_algo"
 #define VLAN_ID_MAX_LEN		4
 
 struct settings_data {
-	const char *sysfs_name;
 	void *data;
 	int (*parse)(struct state *state, int argc, char *argv[]);
 	int (*netlink_get)(struct state *state);
